#include <math.h>
#include <stdio.h>
#include <time.h>
#include <limits.h>

extern void initialise (void);
extern void base_loop (void);
extern void peak_loop (void);
extern void preload (void);
extern const char *compare_loops (void);

extern int base_repeat_count;

#define BASE_FACTOR 250
#define ITERATIONS (BASE_FACTOR * base_repeat_count)
#define REPEATS 10

#define FLOAT_EPSILON 0x1.0p-6f

#define TIMER CLOCK_PROCESS_CPUTIME_ID

int
floats_eq_p (float a, float b)
{
  if (a == b)
    return 1;
  else
    return fabs (a - b) < FLOAT_EPSILON;
}

static void
one_iteration (int i)
{
  while (i-- > 0)
    peak_loop ();
}

int
main (void)
{
  int i;
  struct timespec before, after;
  double time, min_time;
  const char *bad_var;

  initialise ();
  base_loop ();
  peak_loop ();
  bad_var = compare_loops ();
  if (bad_var)
    {
      printf ("%s has the wrong contents\n", bad_var);
      return 1;
    }

  min_time = 0.0;
  for (i = 0; i < REPEATS; i++)
    {
      initialise ();
      preload ();
      one_iteration (1);
      clock_gettime (TIMER, &before);
      preload ();
      one_iteration (ITERATIONS);
      clock_gettime (TIMER, &after);
      time = ((after.tv_sec - before.tv_sec) * 1.0
	      + (after.tv_nsec - before.tv_nsec) * 1.0e-9);
      if (i == 0 || min_time > time)
	min_time = time;
    }
  printf ("%d runs take %gms\n", ITERATIONS, min_time * 1000.0f);
  return 0;
}
